@@ -911,6 +911,12 @@ const completion: Fig.Spec = {
     {
       name: "fetchpost",
       options: [
+        {
+          name: "--payload-tpl",
+        },
+        {
+          name: "--content-type",
+        },
         {
           name: "--new-column",
         },
@@ -2726,6 +2732,9 @@ const completion: Fig.Spec = {
         {
           name: "--dfa-size-limit",
         },
+        {
+          name: "--not-one",
+        },
         {
           name: "--output",
         },
@@ -3572,6 +3581,36 @@ const completion: Fig.Spec = {
         },
       ],
     },
+    {
+      name: "template",
+      options: [
+        {
+          name: "--template",
+        },
+        {
+          name: "--template-file",
+        },
+        {
+          name: "--outfilename",
+        },
+        {
+          name: "--customfilter-error",
+        },
+        {
+          name: "--output",
+        },
+        {
+          name: "--no-headers",
+        },
+        {
+          name: "--delimiter",
+        },
+        {
+          name: ["-h", "--help"],
+          description: "Print help",
+        },
+      ],
+    },
     {
       name: "to",
       subcommands: [
@@ -4193,6 +4232,9 @@ const completion: Fig.Spec = {
         {
           name: "table",
         },
+        {
+          name: "template",
+        },
         {
           name: "to",
           subcommands: [
