@@ -29,7 +29,7 @@ CSV is not sorted by the `--groupby` columns, rows may be
 re-ordered during output due to the buffering of rows
 collected before the first valid value.
 
-For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_fill.rs.
+For examples, see https://github.com/dathere/qsv/blob/master/tests/test_fill.rs.
 
 Usage:
     qsv fill [options] [--] <selection> [<input>]
