@@ -7,7 +7,7 @@ pipe multiple qsv commands together. However, you may want the final result to
 have a specific delimiter or record separator, and this is where 'qsv fmt' is
 useful.
 
-For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_fmt.rs.
+For examples, see https://github.com/dathere/qsv/blob/master/tests/test_fmt.rs.
 
 Usage:
     qsv fmt [options] [<input>]
