@@ -1,6 +1,8 @@
 #![cfg(not(feature = "datapusher_plus"))]
 use std::process;
 
+use serial_test::serial;
+
 use crate::{qcheck, quickcheck::TestResult, workdir::Workdir, Csv, CsvData};
 
 fn no_headers(cmd: &mut process::Command) {
@@ -28,6 +30,7 @@ where
 }
 
 #[test]
+#[serial]
 fn prop_cat_rows() {
     fn p(rows: CsvData) -> bool {
         let expected = rows.clone();
@@ -37,7 +40,9 @@ fn prop_cat_rows() {
             let (rows1, rows2) = rows.split_at(rows.len() / 2);
             (rows1.to_vec(), rows2.to_vec())
         };
+        std::env::set_var("QSV_SKIP_FORMAT_CHECK", "1");
         let got: CsvData = run_cat("cat_rows", "rows", rows1, rows2, no_headers);
+        std::env::remove_var("QSV_SKIP_FORMAT_CHECK");
         rassert_eq!(got, expected)
     }
     qcheck(p as fn(CsvData) -> bool);
@@ -734,8 +739,10 @@ fn cat_rowskey_insertion_order_noheader() {
 }
 
 #[test]
+#[serial]
 fn prop_cat_cols() {
     fn p(rows1: CsvData, rows2: CsvData) -> TestResult {
+        std::env::set_var("QSV_SKIP_FORMAT_CHECK", "1");
         let got: Vec<Vec<String>> = run_cat(
             "cat_cols",
             "columns",
@@ -743,6 +750,7 @@ fn prop_cat_cols() {
             rows2.clone(),
             no_headers,
         );
+        std::env::remove_var("QSV_SKIP_FORMAT_CHECK");
 
         let mut expected: Vec<Vec<String>> = vec![];
         let (rows1, rows2) = (rows1.to_vecs().into_iter(), rows2.to_vecs().into_iter());
