@@ -84,7 +84,8 @@ fn index_outdated_stats() {
             "cv",
             "nullcount",
             "max_precision",
-            "sparsity"
+            "sparsity",
+            "qsv__value",
         ],
         svec![
             "letter",
@@ -106,7 +107,8 @@ fn index_outdated_stats() {
             "",
             "0",
             "",
-            "0"
+            "0",
+            "",
         ],
         svec![
             "number",
@@ -128,7 +130,100 @@ fn index_outdated_stats() {
             "40.8248",
             "0",
             "",
-            "0"
+            "0",
+            "",
+        ],
+        svec![
+            "qsv__rowcount",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "3"
+        ],
+        svec![
+            "qsv__columncount",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "2"
+        ],
+        svec![
+            "qsv__filesize_bytes",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "26"
+        ],
+        svec![
+            "qsv__fingerprint_hash",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "597e467c8f605d260295bc4e059ffb683ac06139c701eda3bf6d5df7d6b1bc8f"
         ],
     ];
 
