@@ -146,7 +146,9 @@ fn sample_seed_url() {
     let mut cmd = wrk.command("sample");
     cmd.args(["--seed", "42"])
         .arg("5")
-        .arg("https://github.com/jqnatividad/qsv/raw/master/resources/test/aliases.csv");
+        .arg("https://github.com/dathere/qsv/raw/master/resources/test/aliases.csv");
+
+    wrk.assert_success(&mut cmd);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
